pub struct AccountState {
pub margin: Margin,
pub positions: HashMap<String, PositionInfo>,
pub open_orders: HashMap<String, OrderState>,
pub leverage_settings: HashMap<String, LeverageSetting>,
}Expand description
Everything a reader might want, exposed as one cheap clone via watch.
The actor publishes a new snapshot after every state-mutating message.
Fields
margin: Marginpositions: HashMap<String, PositionInfo>open_orders: HashMap<String, OrderState>leverage_settings: HashMap<String, LeverageSetting>Trait Implementations
Sourceimpl Clone for AccountState
impl Clone for AccountState
Sourcefn clone(&self) -> AccountState
fn clone(&self) -> AccountState
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for AccountState
impl Debug for AccountState
Sourceimpl Default for AccountState
impl Default for AccountState
Sourcefn default() -> AccountState
fn default() -> AccountState
Returns the “default value” for a type. Read more
Sourceimpl<'de> Deserialize<'de> for AccountState
impl<'de> Deserialize<'de> for AccountState
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more